@@ -1,7 +1,9 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2005  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2006  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
+open Util
+
 let lines_of_channel chan =
   let next () =
     try Some (input_line chan)
@@ -43,16 +45,18 @@ let get_bool = get_generic bool_of_string
 
 let set key value = map := (key, value) :: !map
 
-let iter f = List.iter (fun (k, v) -> f k v) !map
+let fold f init = List.fold_left (fun x (k, v) -> f k v x) init !map
+
+let iter f = fold (fun k v () -> f k v) ()
 
 let read filename =
-  let chan = open_in filename in
-  let lines = List.map words_of_line (lines_of_channel chan) in
-  close_in chan;
-  let enter = function
-    | [ key; value ] -> set key value
-    | [] -> ()
-    | words -> failwith ("malformed line in " ^ filename ^ ": " ^
-			 String.concat " " words)
-  in
-  List.iter enter lines
+  with_channel open_in filename (fun chan ->
+    let lines = List.map words_of_line (lines_of_channel chan) in
+    close_in chan;
+    let enter = function
+      | [ key; value ] -> set key value
+      | [] -> ()
+      | words -> failwith ("malformed line in " ^ filename ^ ": " ^
+			   String.concat " " words)
+    in
+    List.iter enter lines)
